@@ -164,7 +164,6 @@ def hypr_watcher():
 
     client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
     client.connect(f"{hypr_dir}/{his}/.socket2.sock")
-    just_refreshed = False
 
     while True:
         datagram = client.recv(2048)
@@ -177,6 +176,10 @@ def hypr_watcher():
         # print(f"events: {event_names}")
 
         for event_name in event_names:
+            if common_settings["restart-on-display"] and (event_name in ["monitoradded", "monitorremoved"]):
+                print("Received event '{}'; restart in {} ms.".format(event_name, common_settings["restart-delay"]))
+                GLib.timeout_add(common_settings["restart-delay"], restart, priority=GLib.PRIORITY_HIGH)
+
             if event_name in ["activespecial",
                               "activewindow",
                               "activewindowv2",
@@ -191,10 +194,6 @@ def hypr_watcher():
                               "windowtitle",
                               "workspace"]:
 
-                if "activewindow" in event_name and just_refreshed:
-                    just_refreshed = False
-                    break
-
                 # print(f">>> refreshing on {event_name}")
                 monitors, workspaces, clients, activewindow, activeworkspace = h_modules_get_all()
                 for item in common.h_taskbars_list:
@@ -202,9 +201,6 @@ def hypr_watcher():
 
                 for item in common.h_workspaces_list:
                     GLib.timeout_add(0, item.refresh, monitors, workspaces, clients, activewindow, activeworkspace)
-
-                if event_name in ["createworkspace", "destroyworkspace", "focusedmon", "workspace"]:
-                    just_refreshed = True
                 break
 
             elif event_name == "submap":
@@ -324,7 +320,7 @@ def instantiate_content(panel, container, content_list, icons_path=""):
         if item == "hyprland-workspaces":
             if his:
                 if "hyprland-workspaces" in panel:
-                    workspaces = HyprlandWorkspaces(panel["hyprland-workspaces"], monitors, workspaces, clients,
+                    workspaces = HyprlandWorkspaces(panel["hyprland-workspaces"], panel["output"], monitors, workspaces, clients,
                                                     activewindow, activeworkspace, icons_path=icons_path)
                     container.pack_start(workspaces, False, False, panel["items-padding"])
                     common.h_workspaces_list.append(workspaces)
@@ -589,6 +585,8 @@ def main():
     for panel in panels:
         check_key(panel, "output", "")
         check_key(panel, "monitor", "")
+        check_key(panel, "run-if-output-exist", [])
+        check_key(panel, "run-if-output-absent", [])
         if panel["monitor"]:
             try:
                 panel["output"] = common.mon_desc2output_name[panel["monitor"]]
@@ -604,9 +602,17 @@ def main():
                 clones.append(clone)
 
             to_append = to_append + clones
+        else:
+            for output in panel["run-if-output-exist"]:
+                if output not in common.outputs:
+                    to_remove.append(panel)
+            for output in panel["run-if-output-absent"]:
+                if output in common.outputs:
+                    to_remove.append(panel)
 
     for item in to_remove:
-        panels.remove(item)
+        if item in panels:
+            panels.remove(item)
 
     panels = panels + to_append
 
