@@ -14,6 +14,7 @@ import glob
 import gi
 
 import nwg_panel.common
+from nwg_panel.icons import get_icon_name
 
 gi.require_version('GdkPixbuf', '2.0')
 gi.require_version('Gtk', '3.0')
@@ -23,8 +24,6 @@ from gi.repository import Gtk, Gdk, GdkPixbuf
 from shutil import copyfile
 from datetime import datetime
 
-import nwg_panel.common
-
 try:
     import psutil
 except:
@@ -46,108 +45,6 @@ def temp_dir():
     return "/tmp"
 
 
-def get_app_dirs():
-    desktop_dirs = []
-
-    home = os.getenv("HOME")
-    xdg_data_home = os.getenv("XDG_DATA_HOME")
-    xdg_data_dirs = os.getenv("XDG_DATA_DIRS") if os.getenv("XDG_DATA_DIRS") else "/usr/local/share/:/usr/share/"
-
-    if xdg_data_home:
-        desktop_dirs.append(os.path.join(xdg_data_home, "applications"))
-    else:
-        if home:
-            desktop_dirs.append(os.path.join(home, ".local/share/applications"))
-
-    for d in xdg_data_dirs.split(":"):
-        desktop_dirs.append(os.path.join(d, "applications"))
-
-    # Add flatpak dirs if not found in XDG_DATA_DIRS
-    flatpak_dirs = [os.path.join(home, ".local/share/flatpak/exports/share/applications"),
-                    "/var/lib/flatpak/exports/share/applications"]
-    for d in flatpak_dirs:
-        if d not in desktop_dirs:
-            desktop_dirs.append(d)
-
-    return desktop_dirs
-
-
-# def map_odd_desktop_files():
-#     name2icon_dict = {}
-#     for d in nwg_panel.common.app_dirs:
-#         if os.path.exists(d):
-#             for path in os.listdir(d):
-#                 if os.path.isfile(os.path.join(d, path)):
-#                     if path.endswith(".desktop") and path.count(".") > 1:
-#                         try:
-#                             content = load_text_file(os.path.join(d, path))
-#                         except Exception as e:
-#                             eprint(e)
-#                         if content:
-#                             for line in content.splitlines():
-#                                 if line.startswith("[") and not line == "[Desktop Entry]":
-#                                     break
-#                                 if line.upper().startswith("ICON="):
-#                                     icon = line.split("=")[1]
-#                                     name2icon_dict[path] = icon
-#                                     break
-#
-#     return name2icon_dict
-
-
-def get_icon_name(app_name):
-    if not app_name:
-        return ""
-    # the name might've been already found in .desktop files and stored
-    if app_name in nwg_panel.common.app_name2icon_name:
-        return nwg_panel.common.app_name2icon_name[app_name]
-
-    # GIMP returns "app_id": null and for some reason "class": "Gimp-2.10" instead of just "gimp".
-    # Until the GTK3 version is released, let's make an exception for GIMP.
-    if "GIMP" in app_name.upper():
-        return "gimp"
-
-    for d in nwg_panel.common.app_dirs:
-        # This will work if the .desktop file name is app_id.desktop or wm_class.desktop
-        path = os.path.join(d, "{}.desktop".format(app_name))
-        content = None
-        if os.path.isfile(path):
-            content = load_text_file(path)
-        elif os.path.isfile(path.lower()):
-            content = load_text_file(path.lower())
-        if content:
-            for line in content.splitlines():
-                if line.upper().startswith("ICON"):
-                    icon_name = line.split("=")[1]
-                    # store for further use
-                    nwg_panel.common.app_name2icon_name[app_name] = icon_name
-                    return icon_name
-
-        # Otherwise, we need to search all .desktop files
-        if os.path.isdir(d):
-            for a_file in os.listdir(d):
-                if app_name.lower() in a_file.lower():
-                    path = os.path.join(d, a_file)
-                    content = load_text_file(path)
-                    if content:
-                        for line in content.splitlines():
-                            if line.upper().startswith("ICON"):
-                                icon_name = line.split("=")[1]
-                                # store for further use
-                                nwg_panel.common.app_name2icon_name[app_name] = icon_name
-                                return icon_name
-
-    # Search the dictionary made of .desktop files that use "reverse DNS"-style names, prepared on startup.
-    # see: https://github.com/nwg-piotr/nwg-panel/issues/64
-    # !!! This was case-sensitive, boo :/
-    # for key in nwg_panel.common.name2icon_dict.keys():
-    #     if app_name in key.split("."):
-    #         return nwg_panel.common.name2icon_dict[key]
-
-    # if all above fails
-    return app_name
-
-
 def local_dir():
     local_dir = os.path.join(os.path.join(os.getenv("HOME"), ".local/share/nwg-panel"))
     if not os.path.isdir(local_dir):
@@ -277,36 +174,44 @@ def num_active_outputs(outputs):
     return a
 
 
-def list_outputs(sway=False, tree=None, silent=False):
+def list_outputs(sway=False, silent=False):
     """
     Get output names and geometry from i3 tree, assign to Gdk.Display monitors.
     :return: {"name": str, "x": int, "y": int, "width": int, "height": int, "monitor": Gkd.Monitor}
     """
     outputs_dict = {}
     if sway:
+        try:
+            from i3ipc import Connection
+        except ModuleNotFoundError:
+            print("'python-i3ipc' package required on sway, terminating")
+            sys.exit(1)
+
+        i3 = Connection()
+
         if not silent:
             print("Running on sway")
-        if not tree:
-            tree = nwg_panel.common.i3.get_tree()
-        for item in tree:
-            if item.type == "output" and not item.name.startswith("__"):
-                outputs_dict[item.name] = {"x": item.rect.x,
-                                           "y": item.rect.y,
-                                           "width": item.rect.width,
-                                           "height": item.rect.height,
-                                           "monitor": None}
+        outputs = i3.get_outputs()
+        for item in outputs:
+            outputs_dict[item.name] = {"x": item.rect.x,
+                                       "y": item.rect.y,
+                                       "width": item.rect.width,
+                                       "height": item.rect.height,
+                                       "description": f"{item.make} {item.model} {item.serial}",
+                                       "monitor": None}
 
     elif os.getenv('HYPRLAND_INSTANCE_SIGNATURE') is not None:
         if not silent:
             print("Running on Hyprland")
         cmd = "hyprctl -j monitors"
-        output = subprocess.check_output(cmd, shell=True).decode("utf-8").strip()
-        monitors = json.loads(output)
-        for item in monitors:
+        result = subprocess.check_output(cmd, shell=True).decode("utf-8").strip()
+        outputs = json.loads(result)
+        for item in outputs:
             outputs_dict[item["name"]] = {"x": item["x"],
                                           "y": item["y"],
                                           "width": int(item["width"] / item["scale"]),
                                           "height": int(item["height"] / item["scale"]),
+                                          "description": item["description"],
                                           "monitor": None}
             # swap for rotated displays
             if item["transform"] in [1, 3, 5, 7]:
@@ -319,10 +224,11 @@ def list_outputs(sway=False, tree=None, silent=False):
         if nwg_panel.common.commands["wlr-randr"]:
             lines = subprocess.check_output("wlr-randr", shell=True).decode("utf-8").strip().splitlines()
             if lines:
-                name, w, h, x, y, transform, scale = None, None, None, None, None, None, 1.0
+                name, description, w, h, x, y, transform, scale = None, None, None, None, None, None, None, 1.0
                 for line in lines:
                     if not line.startswith(" "):
                         name = line.split()[0]
+                        description = line.split()[1]
                     elif "current" in line:
                         w_h = line.split()[0].split('x')
                         w = int(w_h[0])
@@ -348,6 +254,7 @@ def list_outputs(sway=False, tree=None, silent=False):
                                                   'width': int(w / scale),
                                                   'height': int(h / scale),
                                                   'transform': transform,
+                                                  'description': description,
                                                   'monitor': None}
                         else:
                             outputs_dict[name] = {'name': name,
@@ -356,22 +263,36 @@ def list_outputs(sway=False, tree=None, silent=False):
                                                   'width': int(h / scale),
                                                   'height': int(w / scale),
                                                   'transform': transform,
+                                                  'description': description,
                                                   'scale': scale,
                                                   'monitor': None}
+                        # Each monitor only has a single transform, this is to avoid parsing the same monitor multiple times
+                        # Disabled monitors don't have transforms.
+                        # Gdk doesn't report disabled monitors, not filtering them would cause crashes
+                        transform = None
         else:
             print("'wlr-randr' command not found, terminating")
             sys.exit(1)
 
+    # We used to assign Gdk.Monitor to output on the basis of x and y coordinates, but it no longer works,
+    # starting from gtk3-1:3.24.42: all monitors have x=0, y=0. This is most likely a bug, but from now on
+    # we must rely on gdk monitors order. Hope it's going to work.
+    monitors = []
     display = Gdk.Display.get_default()
     for i in range(display.get_n_monitors()):
         monitor = display.get_monitor(i)
-        geometry = monitor.get_geometry()
+        monitors.append(monitor)
 
-        for key in outputs_dict:
-            if int(outputs_dict[key]["x"]) == geometry.x and int(outputs_dict[key]["y"]) == geometry.y:
-                outputs_dict[key]["monitor"] = monitor
+    for key, monitor in zip(outputs_dict.keys(), monitors):
+        outputs_dict[key]["monitor"] = monitor
 
-    return outputs_dict
+    # map monitor descriptions to output names
+    mon_desc2output_name = {}
+    for key in outputs_dict:
+        if "description" in outputs_dict[key]:
+            mon_desc2output_name[outputs_dict[key]["description"]] = key
+
+    return outputs_dict, mon_desc2output_name
 
 
 def check_key(dictionary, key, default_value):
@@ -475,8 +396,11 @@ def list_sink_inputs():
     """
     Thanks to @fm16191 for https://github.com/fm16191/pactl-json-parser
     """
-    p = subprocess.run("pactl list sink-inputs".split(), capture_output=True)
-    result = p.stdout.decode()
+    # This used to return localized output, returning unexpected values #327
+    # p = subprocess.run("pactl list sink-inputs".split(), capture_output=True)
+    # result = p.stdout.decode()
+
+    result = cmd2string("pactl list sink-inputs")
 
     # Syntax fix
     result = result.replace(u"\xa0", " ")  # Broken encoding for spaces
@@ -529,11 +453,14 @@ def list_sinks():
                 for line in lines:
                     details = line.split()
                     name = details[1][1:-1]
-                    desc = " ".join(details[2:])[1:-1]
-                    sinks.append({"name": name, "desc": desc})
+                    desc = " ".join(details[3:])[1:-1]
+                    sink = {"name": name, "desc": desc, "running": True if "Running" in line else False}
+                    sinks.append(sink)
+
         except Exception as e:
             eprint(e)
-    if nwg_panel.common.commands["pactl"]:
+
+    elif nwg_panel.common.commands["pactl"]:
         try:
             output = cmd2string("pactl list sinks")
             if output:
@@ -550,6 +477,8 @@ def list_sinks():
                             sink.update({"name": line.split(": ")[1]})
                         elif line.lower().startswith("description"):
                             sink.update({"desc": line.split(": ")[1]})
+                        elif line.lower().startswith("state"):
+                            sink.update({"running": True if "RUNNING" in line else False})
                 if sink:
                     sinks.append(sink)
         except Exception as e:
@@ -820,9 +749,14 @@ def load_shell_data():
 
 
 def hyprctl(cmd, buf_size=2048):
+    # /tmp/hypr moved to $XDG_RUNTIME_DIR/hypr in #5788
+    xdg_runtime_dir = os.getenv("XDG_RUNTIME_DIR")
+    hypr_dir = f"{xdg_runtime_dir}/hypr" if xdg_runtime_dir and os.path.isdir(
+        f"{xdg_runtime_dir}/hypr") else "/tmp/hypr"
+
     s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
     try:
-        s.connect("/tmp/hypr/{}/.socket.sock".format(os.getenv("HYPRLAND_INSTANCE_SIGNATURE")))
+        s.connect(f"{hypr_dir}/{os.getenv('HYPRLAND_INSTANCE_SIGNATURE')}/.socket.sock")
         s.send(cmd.encode("utf-8"))
 
         output = b""
@@ -833,7 +767,7 @@ def hyprctl(cmd, buf_size=2048):
             else:
                 break
         s.close()
-        return output.decode('utf-8')
+        return output.decode('utf-8', errors='replace')
     except Exception as e:
         eprint(f"hyprctl: {e}")
         return ""
@@ -855,6 +789,14 @@ def h_list_workspaces():
     except Exception as e:
         eprint(e)
         return {}
+    
+def h_list_workspace_rules():
+    reply = hyprctl("j/workspacerules")
+    try:
+        return json.loads(reply)
+    except Exception as e:
+        eprint(e)
+        return {}
 
 
 def h_list_clients():
@@ -875,16 +817,55 @@ def h_get_activewindow():
         return {}
 
 
+def h_get_active_workspace():
+    reply = hyprctl("j/activeworkspace")
+    try:
+        return json.loads(reply)
+    except Exception as e:
+        eprint(e)
+        return {}
+
+
 def h_modules_get_all():
-    return h_list_monitors(), h_list_workspaces(), h_list_clients(), h_get_activewindow()
+    return h_list_monitors(), h_list_workspaces(), h_list_clients(), h_get_activewindow(), h_get_active_workspace()
 
 
 def cmd_through_compositor(cmd):
+    cmd = cmd.replace("\"", "'")
     cs_file = os.path.join(get_config_dir(), "common-settings.json")
     common_settings = load_json(cs_file)
-    if "run-through-compositor" not in common_settings or common_settings["run-through-compositor"] :
+    if "run-through-compositor" not in common_settings or common_settings["run-through-compositor"]:
         if os.getenv("SWAYSOCK"):
-            cmd = f"swaymsg exec '{cmd}'"
+            if os.getenv("XDG_SESSION_DESKTOP") and "miracle-wm" in os.getenv("XDG_SESSION_DESKTOP"):
+                cmd = f'miraclemsg exec "{cmd}"'
+            else:
+                cmd = f'swaymsg exec "{cmd}"'
         elif os.getenv("HYPRLAND_INSTANCE_SIGNATURE"):
-            cmd = f"hyprctl dispatch exec '{cmd}'"
+            cmd = f'hyprctl dispatch exec "{cmd}"'
     return cmd
+
+
+def load_resource(package, resource_name):
+    try:
+        import importlib.resources as resources
+        with resources.open_binary(package, resource_name) as resource_file:
+            return resource_file.read()
+    except:
+        pass
+
+    try:
+        import importlib.util
+        spec = importlib.util.find_spec(package)
+        if spec is not None and spec.loader is not None and hasattr(spec.loader, 'get_data'):
+            return spec.loader.get_data(resource_name)
+    except Exception:
+        pass
+
+    try:
+        import pkgutil
+        data = pkgutil.get_data(package, resource_name)
+        if data is None:
+            raise FileNotFoundError(f"Resource {resource_name} not found in package {package}.")
+        return data
+    except ImportError as e:
+        raise ImportError("Failed to load the resource using any available method.") from e
